.media {
    position: relative;
    display: inline-block;
}

.video_arrow {
    position: absolute;
    border: none;
    width: 40%;
    top: 0;
    bottom: 0;
    left: 0;
    right: 0;
    margin: auto;
}

#video_vid {
    text-align: center;
}

video {
    max-width: 100%;
}
