@import url("basic.css");

body{
    background: white;
}

a {
    color: black;
}

.title {
    display: none;
}

.title h1{
    margin-top: .1em;
}

.inline_enum ul{
    margin-left: 0;
    padding-left: 0;
    display: inline;
}

.inline_enum ul li {
    margin-left: 0;
    padding-left: 2px;
    border: none;
    list-style: none;
    display: inline;
}

#breadcrumbs {
    padding: 3px;
    margin-bottom: 25px;
    font-size: small;
}

#breadcrumbs ul li:after {
    content: "\0020 \0020 \0020 \00BB \0020";
}

#breadcrumbs ul li.bc_current:after {
    content: " ";
}

#osize_links ul li:after {
    content: "   |";
}

#osize_links ul li:last-child:after {
    content: " ";
}

#onum_links ul li:after {
    content: "   |";
}

#onum_links ul li:last-child:after {
    content: " ";
}

#osize_links {
    font-size: small;
    text-align:center;
    margin: 1em;
}

#onum_links{
    font-size: small;
    text-align:center;
}

div.subgal_link {
    float: left;
    padding: 1em;
    width: 50em;
}

div.subgal_image {
    float: left;
    margin-right: 2em;
    margin-left: 4em;
    width:180px;
}

div.subgal_image img {
    margin: 0;
    border: none;
}

div.subgal_description p{
    text-align:justify;
}

.media_links{
    padding: 2em;
}

img.media{
    border: solid black 1px;
    margin: 1em;
}

.prevnext{
    text-align: center;
    margin: 2em;
    font-size: x-small;
}

.prevnext a{
    display: block;
}

#image{
    margin-left: 5px;
}

#image_img {
    float: left;
    padding-right: 1em;
    max-width: 70%;
    padding-right: 1em;
}

#image img {
    height: auto;
    border: solid black 1px;
    max-width: 100%;
}

#image_caption {
    padding-left: 70%;
    font-size: x-small;
}

#image_caption .image_comment{
    font-size: x-large;
    margin-bottom: 1em;
}

.image_caption_tech {
    display: none;
}

#lazygalfooter{
    padding-top: 3em;
    clear: both;
    font-size: x-small;
}

@media only screen and (max-width: 700px){
    body {
        margin: 0;
        padding: 0;
    }

    #osize_links {
        display: none;
    }

    #breadcrumbs {
        margin: 2px;
        padding: 0;
    }

    .prevnext {
        margin: 0.1em;
    }

    .prevnext img {
        vertical-align: middle;
        max-height: 10%;
        width: auto;
    }

    #image_caption .image_comment{
        font-size: small;
    }

    #lazygalfooter{
        padding: 1px;
    }
}

@media only screen and (max-width: 700px) and (orientation: portrait){
    #image_caption {
        padding: 5px;
    }

    #image_img {
        float: none;
        max-width: 100%;
    }

    .prevnext img, .prevnext a {
        display: inline;
    }
}

/*
 * vim: ts=4 sw=4 expandtab
 */
